@@ -15,9 +15,13 @@ hook_plot_md = function(x, options) {
     }
     if (office_output) {
       if (options$fig.align != 'default') {
-        warning('Chunk options fig.align is not supported for ', to, ' output')
+        warn_options_unsupported('fig.align', to)
         options$fig.align = 'default'
       }
+      if (!is.null(options$fig.alt)) {
+        warn_options_unsupported('fig.alt', to)
+        options$fig.alt = NULL
+      }
       return(hook_plot_md_pandoc(x, options))
     }
   }
@@ -29,8 +33,8 @@ hook_plot_md = function(x, options) {
 need_special_plot_hook = function(options) {
   opts = opts_chunk$get(default = TRUE)
   for (i in c(
-    'out.width', 'out.height', 'out.extra',
-    'fig.align', 'fig.subcap', 'fig.env', 'fig.scap'
+    'out.width', 'out.height', 'out.extra', 'fig.align', 'fig.subcap',
+    'fig.env', 'fig.scap', 'fig.alt'
   )) if (!identical(options[[i]], opts[[i]])) return(TRUE)
   FALSE
 }
@@ -148,6 +152,15 @@ render_markdown = function(strict = FALSE, fence_char = '`') {
 #' @export
 hooks_markdown = function(strict = FALSE, fence_char = '`') {
   fence = paste(rep(fence_char, 3), collapse = '')
+  update_fence = function(x) {
+    r = paste0('\n', fence_char, '{3,}')
+    if (grepl(r, x)) {
+      l = attr(gregexpr(r, x)[[1]], 'match.length')
+      l = max(l)
+      if (l >= 4) fence = paste(rep(fence_char, l), collapse = '')
+    }
+    fence
+  }
   # four spaces lead to <pre></pre>
   hook.t = function(x, options, attr = NULL, class = NULL) {
     # this code-block duplicated from hook.t()
@@ -155,12 +168,7 @@ hooks_markdown = function(strict = FALSE, fence_char = '`') {
       paste('\n', indent_block(x), '', sep = '\n')
     } else {
       x = one_string(c('', x))
-      r = paste0('\n', fence_char, '{3,}')
-      if (grepl(r, x)) {
-        l = attr(gregexpr(r, x)[[1]], 'match.length')
-        l = max(l)
-        if (l >= 4) fence = paste(rep(fence_char, l), collapse = '')
-      }
+      fence = update_fence(x)
       paste0('\n\n', fence, block_attr(attr, class), x, fence, '\n\n')
     }
   }
@@ -175,6 +183,7 @@ hooks_markdown = function(strict = FALSE, fence_char = '`') {
     if (language == 'node') language = 'javascript'
     if (!options$highlight) language = 'text'
     attrs = block_attr(options$attr.source, options$class.source, language)
+    fence = update_fence(one_string(c('', x)))
     paste0('\n\n', fence, attrs, '\n', x, fence, '\n\n')
   }
   list(
