@@ -4,8 +4,8 @@
 #' list of patterns, evaluates the code and writes the output in another file.
 #' It can also tangle R source code from the input document (\code{purl()} is a
 #' wrapper to \code{knit(..., tangle = TRUE)}). The \code{knitr.purl.inline}
-#' option can be used to also tangle the code of inline expressions (disabled
-#' by default).
+#' option can be used to also tangle the code of inline expressions (disabled by
+#' default).
 #'
 #' For most of the time, it is not necessary to set any options outside the
 #' input document; in other words, a single call like
@@ -34,9 +34,9 @@
 #' \code{apat$rnw}, \samp{tex} uses the list \code{apat$tex}, \samp{brew} uses
 #' \code{apat$brew} and HTML files use \code{apat$html}; for unkown extensions,
 #' the content of the input document is matched against all pattern lists to
-#' automatically determine which pattern list is being used. You can also manually
-#' set the pattern list using the \code{\link{knit_patterns}} object or the
-#' \code{\link{pat_rnw}} series functions in advance and \pkg{knitr} will
+#' automatically determine which pattern list is being used. You can also
+#' manually set the pattern list using the \code{\link{knit_patterns}} object or
+#' the \code{\link{pat_rnw}} series functions in advance and \pkg{knitr} will
 #' respect the setting.
 #'
 #' According to the output format (\code{opts_knit$get('out.format')}), a set of
@@ -57,15 +57,15 @@
 #' etc.
 #' @param input Path to the input file.
 #' @param output Path to the output file for \code{knit()}. If \code{NULL}, this
-#'   function will try to guess a default, which will be under the current working
-#'   directory.
+#'   function will try to guess a default, which will be under the current
+#'   working directory.
 #' @param tangle Boolean; whether to tangle the R code from the input file (like
-#'   \code{\link[utils]{Stangle}}).
-#' @param text A character vector. This is an alternative way to provide the input
-#'   file.
+#'   \code{utils::\link{Stangle}}).
+#' @param text A character vector. This is an alternative way to provide the
+#'   input file.
 #' @param quiet Boolean; suppress the progress bar and messages?
-#' @param envir Environment in which code chunks are to be evaluated,
-#'   for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
+#' @param envir Environment in which code chunks are to be evaluated, for
+#'   example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
 #'   \code{\link{globalenv}()}).
 #' @param encoding Encoding of the input file; always assumed to be UTF-8 (i.e.,
 #'   this argument is effectively ignored).
@@ -82,20 +82,20 @@
 #'   \code{\link{opts_knit}$set(root.dir = ...)} so all paths in code chunks are
 #'   relative to this \code{root.dir}. It is not recommended to change the
 #'   working directory via \code{\link{setwd}()} in a code chunk, because it may
-#'   lead to terrible consequences (e.g. figure and cache files may be written to
-#'   wrong places). If you do use \code{setwd()}, please note that \pkg{knitr}
-#'   will always restore the working directory to the original one. Whenever you
-#'   feel confused, print \code{getwd()} in a code chunk to see what the working
-#'   directory really is.
+#'   lead to terrible consequences (e.g. figure and cache files may be written
+#'   to wrong places). If you do use \code{setwd()}, please note that
+#'   \pkg{knitr} will always restore the working directory to the original one.
+#'   Whenever you feel confused, print \code{getwd()} in a code chunk to see
+#'   what the working directory really is.
 #'
 #'   If the \code{output} argument is a file path, it is strongly recommended to
 #'   be in the current working directory (e.g. \file{foo.tex} instead of
 #'   \file{somewhere/foo.tex}), especially when the output has external
 #'   dependencies such as figure files. If you want to write the output to a
-#'   different directory, it is recommended to set the working directory to
-#'   that directory before you knit a document. For example, if the source
-#'   document is \file{foo.Rmd} and the expected output is \file{out/foo.md},
-#'   you can write \code{setwd('out/'); knit('../foo.Rmd')} instead of
+#'   different directory, it is recommended to set the working directory to that
+#'   directory before you knit a document. For example, if the source document
+#'   is \file{foo.Rmd} and the expected output is \file{out/foo.md}, you can
+#'   write \code{setwd('out/'); knit('../foo.Rmd')} instead of
 #'   \code{knit('foo.Rmd', 'out/foo.md')}.
 #'
 #'   N.B. There is no guarantee that the R script generated by \code{purl()} can
@@ -105,7 +105,9 @@
 #'   you want to reproduce the computation in a report generated by
 #'   \code{knit()}, be sure to use \code{knit()}, instead of merely executing
 #'   the R script generated by \code{purl()}. This seems to be obvious, but some
-#'   people \href{http://bit.ly/SnLi6h}{just do not get it}.
+#'   people
+#'   \href{https://stat.ethz.ch/pipermail/r-devel/2014-May/069113.html}{do not
+#'   get it}.
 #' @export
 #' @references Package homepage: \url{https://yihui.org/knitr/}. The \pkg{knitr}
 #'   \href{https://yihui.org/knitr/demo/manual/}{main manual}: and
@@ -134,7 +136,7 @@ knit = function(
     # in child mode, input path needs to be adjusted
     if (in.file && !is_abs_path(input)) {
       input = paste0(opts_knit$get('child.path'), input)
-      input = file.path(input_dir(TRUE), input)
+      input = file.path(input_dir(), input)
     }
     # respect the quiet argument in child mode (#741)
     optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
@@ -186,13 +188,7 @@ knit = function(
     knit_concord$set(infile = input, outfile = output)
   }
 
-  if (is.null(text)) {
-    text = readLines(input, encoding = 'UTF-8', warn = FALSE)
-    if (!is_utf8(text)) warning(
-      'The file "', input, '" must be encoded in UTF-8. Please see ',
-      'https://yihui.org/en/2018/11/biggest-regret-knitr/ for more info.'
-    )
-  } else text = split_lines(text) # make sure each element is one line
+  text = if (is.null(text)) xfun::read_utf8(input) else split_lines(text)
   if (!length(text)) {
     if (is.character(output)) file.create(output)
     return(output) # a trivial case: create an empty file and exit
@@ -233,7 +229,7 @@ knit = function(
       # in child mode, strip off the YAML metadata in Markdown if exists
       if (grepl('^---\\s*$', text[1])) {
         i = grep('^---\\s*$', text)
-        if (length(i) >= 2) text = text[-(1:i[2])]
+        if (length(i) >= 2) text[1:i[2]] = ''
       }
     } else {
       params = knit_params(text)
@@ -294,8 +290,11 @@ process_file = function(text, output) {
   wd = getwd()
   for (i in 1:n) {
     if (!is.null(.knitEnv$terminate)) {
-      res[i] = one_string(.knitEnv$terminate)
-      knit_exit(NULL)
+      if (!child_mode() || !.knitEnv$terminate_fully) {
+        # reset the internal variable `terminate` in the top parent
+        res[i] = one_string(.knitEnv$terminate)
+        knit_exit(NULL, NULL)
+      }
       break  # must have called knit_exit(), so exit early
     }
     if (progress) {
@@ -409,15 +408,19 @@ knit_child = function(..., options = NULL, envir = knit_global()) {
 #'   \code{knit()} so far. By default, this is \samp{\end{document}} for LaTeX
 #'   output, and \samp{</body></html>} for HTML output, to make the output
 #'   document complete. For other types of output, it is an empty string.
+#' @param fully Whether to fully exit the knitting process if \code{knit_exit()}
+#'   is called from a child document. If \code{FALSE}, only exit the knitting
+#'   process of the child document.
 #' @return Invisible \code{NULL}. An internal signal is set up (as a side
 #'   effect) to notify \code{knit()} to quit as if it had reached the end of the
 #'   document.
 #' @export
 #' @examples # see https://github.com/yihui/knitr-examples/blob/master/096-knit-exit.Rmd
-knit_exit = function(append) {
+knit_exit = function(append, fully = TRUE) {
   if (missing(append)) append = if (out_format(c('latex', 'sweave', 'listings')))
     '\\end{document}' else if (out_format('html')) '</body>\n</html>' else ''
   .knitEnv$terminate = append # use this terminate variable to notify knit()
+  .knitEnv$terminate_fully = fully
   invisible()
 }
 
@@ -425,25 +428,39 @@ knit_log = new_defaults()  # knitr log for errors, warnings and messages
 
 #' Wrap evaluated results for output
 #'
-#' @param x output from \code{\link[evaluate]{evaluate}}
-#' @param options List of options used to control output
-#' @noRd
-wrap = function(x, options = list(), ...) {
-  UseMethod('wrap', x)
+#' This function is mainly for internal use: it is called on each part of the
+#' output of the code chunk (code, messages, text output, and plots, etc.) after
+#' all statements in the code chunk have been evaluated, and will sew these
+#' pieces of output together into a character vector.
+#' @param x Output from \code{evaluate::\link{evaluate}()}.
+#' @param options A list of chunk options used to control output.
+#' @param ... Other arguments to pass to methods.
+#' @export
+sew = function(x, options = list(), ...) {
+  UseMethod('sew', x)
+}
+
+# TODO: see if we can remove this function in the future
+wrap = function(...) {
+  warning2(
+    'The internal function knitr:::wrap() has been deprecated. Please use the ',
+    'exported function knitr::sew() instead.'
+  )
+  sew(...)
 }
 
 #' @export
-wrap.list = function(x, options = list()) {
+sew.list = function(x, options = list(), ...) {
   if (length(x) == 0L) return(x)
-  lapply(x, wrap, options)
+  lapply(x, sew, options, ...)
 }
 
 # ignore unknown classes
 #' @export
-wrap.default = function(x, options) return()
+sew.default = function(x, options, ...) return()
 
 #' @export
-wrap.character = function(x, options) {
+sew.character = function(x, options, ...) {
   if (options$results == 'hide') return()
   if (output_asis(x, options)) {
     if (!out_format('latex')) return(x)  # latex output still need a tweak
@@ -454,7 +471,7 @@ wrap.character = function(x, options) {
 # If you provide a custom print function that returns a character object of
 # class 'knit_asis', it will be written as is.
 #' @export
-wrap.knit_asis = function(x, options, inline = FALSE) {
+sew.knit_asis = function(x, options, inline = FALSE, ...) {
   m = attr(x, 'knit_meta')
   knit_meta_add(m, if (missing(options)) '' else options$label)
   if (!missing(options)) {
@@ -479,10 +496,10 @@ wrap.knit_asis = function(x, options, inline = FALSE) {
 }
 
 #' @export
-wrap.source = function(x, options) {
+sew.source = function(x, options, ...) {
   if (isFALSE(options$echo)) return()
   src = sub('\n$', '', x$src)
-  if (!options$collapse && options$strip.white) src = strip_white(src)
+  if (options$strip.white) src = strip_white(src)
   if (is_blank(src)) return()  # an empty chunk
   knit_hooks$get('source')(src, options)
 }
@@ -511,26 +528,26 @@ msg_sanitize = function(message, type) {
 }
 
 #' @export
-wrap.warning = function(x, options) {
+sew.warning = function(x, options, ...) {
   call = if (is.null(x$call)) '' else {
     call = deparse(x$call)[1]
     if (call == 'eval(expr, envir, enclos)') '' else paste(' in', call)
   }
-  msg_wrap(sprintf('Warning%s: %s', call, x$message), 'warning', options)
+  msg_wrap(sprintf('Warning%s: %s', call, conditionMessage(x)), 'warning', options)
 }
 
 #' @export
-wrap.message = function(x, options) {
-  msg_wrap(paste(x$message, collapse = ''), 'message', options)
+sew.message = function(x, options, ...) {
+  msg_wrap(paste(conditionMessage(x), collapse = ''), 'message', options)
 }
 
 #' @export
-wrap.error = function(x, options) {
+sew.error = function(x, options, ...) {
   msg_wrap(as.character(x), 'error', options)
 }
 
 #' @export
-wrap.recordedplot = function(x, options) {
+sew.recordedplot = function(x, options, ...) {
   # figure number sequence for multiple plots
   fig.cur = plot_counter()
   options$fig.cur = fig.cur # put fig num in options
@@ -552,7 +569,7 @@ wrap.recordedplot = function(x, options) {
 }
 
 #' @export
-wrap.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE) {
+sew.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE, ...) {
   if (options$fig.show == 'hide') return('')
   # remove the automatically set out.width when fig.retina is set, otherwise the
   # size of external images embedded via include_graphics() will be set to
@@ -563,6 +580,7 @@ wrap.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE)
     if (length(w1) * length(w2) == 1 && is.numeric(w1) && w1 == w2)
       options['out.width'] = list(NULL)
   }
+  options$fig.num = options$fig.num %n% length(x)
   dpi = attr(x, 'dpi') %n% options$dpi
   hook = knit_hooks$get('plot')
   paste(unlist(lapply(seq_along(x), function(i) {
@@ -573,8 +591,17 @@ wrap.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE)
   })), collapse = '')
 }
 
+# TODO: remove this in the future
+wrap.knit_image_paths = function(...) {
+  warning2(
+    "The internal function knitr:::wrap.knit_image_paths has been deprecated. ",
+    "Please use knitr::sew() on an object of the class 'knit_image_paths' instead."
+  )
+  sew.knit_image_paths(...)
+}
+
 #' @export
-wrap.html_screenshot = function(x, options = opts_chunk$get(), inline = FALSE) {
+sew.html_screenshot = function(x, options = opts_chunk$get(), inline = FALSE, ...) {
   ext = x$extension
   in_base_dir({
     i = plot_counter()
@@ -599,12 +626,12 @@ run_hook_plot = function(x, options) {
 }
 
 #' @export
-wrap.knit_embed_url = function(x, options = opts_chunk$get(), inline = FALSE) {
+sew.knit_embed_url = function(x, options = opts_chunk$get(), inline = FALSE, ...) {
   options$fig.cur = plot_counter()
   options = reduce_plot_opts(options)
   if (length(extra <- options$out.extra)) extra = paste('', extra, collapse = '')
   add_html_caption(options, sprintf(
-    '<iframe src="%s" width="%s" height="%s"%s></iframe>',
+    '<iframe src="%s" width="%s" height="%s" data-external="1"%s></iframe>',
     escape_html(x$url), options$out.width %n% '100%', x$height %n% '400px',
     extra %n% ''
   ))
@@ -672,6 +699,9 @@ knit_print.default = function(x, ..., inline = FALSE) {
 #' @export
 knit_print.knit_asis = function(x, ...) x
 
+#' @export
+knit_print.knit_asis_url = function(x, ...) x
+
 #' @rdname knit_print
 #' @export
 normal_print = default_handlers$value
