@@ -12,15 +12,12 @@ split_file = function(lines, set.preamble = TRUE, patterns = knit_patterns$get()
     set_preamble(lines, patterns)  # prepare for tikz option 'standAlone'
   }
 
-  blks = grepl(chunk.begin, lines)
-  txts = filter_chunk_end(blks, grepl(chunk.end, lines))
-  # tmp marks the starting lines of a code/text chunk by TRUE
-  tmp = blks | head(c(TRUE, txts), -1)
+  markdown_mode = identical(patterns, all_patterns$md)
+  i = group_indices(grepl(chunk.begin, lines), grepl(chunk.end, lines), lines, markdown_mode)
+  groups = unname(split(lines, i))
 
-  groups = unname(split(lines, cumsum(tmp)))
   if (set.preamble)
     knit_concord$set(inlines = sapply(groups, length)) # input line numbers for concordance
-  markdown_mode = identical(patterns, all_patterns$md)
 
   # parse 'em all
   lapply(groups, function(g) {
@@ -88,8 +85,16 @@ parse_block = function(code, header, params.src, markdown_mode = out_format('mar
     params = gsub('^\\s*,\\s*', '', params)
   }
 
-  params.src = params
-  params = parse_params(params.src)
+  # for quarto, preserve the actual original params.src and do not remove the engine
+  if (!(is_quarto <- !is.null(opts_knit$get('quarto.version')))) params.src = params
+  params = parse_params(params)
+
+  # merge with possible chunk options written as (YAML or CSV) metadata in
+  # chunk, and remove metadata from code body
+  parts = partition_chunk(engine, code)
+  params = merge_list(params, parts$options)
+  code = parts$code
+
   # remove indent (and possibly markdown blockquote >) from code
   if (nzchar(spaces <- gsub('^([\t >]*).*', '\\1', header))) {
     params$indent = spaces
@@ -123,6 +128,18 @@ parse_block = function(code, header, params.src, markdown_mode = out_format('mar
       dep_list$set(setNames(list(c(dep_list$get(i), label)), i))
   }
 
+  # for quarto only
+  if (is_quarto) {
+    params$original.params.src = params.src
+    params$chunk.echo = isTRUE(params[['echo']])
+    params$yaml.code = parts$src
+    attr(params, 'quarto_options') = c('original.params.src', 'chunk.echo', 'yaml.code')
+    # alias 'warning' explicitly set in chunk metadata to the 'message' option
+    if (!is.null(parts$options[['warning']])) {
+      params$message = parts$options[['warning']]
+    }
+  }
+
   structure(list(params = params, params.src = params.src), class = 'block')
 }
 
@@ -133,9 +150,9 @@ unnamed_chunk = function(prefix = NULL, i = chunk_counter()) {
 }
 
 # parse params from chunk header
-parse_params = function(params) {
+parse_params = function(params, label = TRUE) {
 
-  if (params == '') return(list(label = unnamed_chunk()))
+  if (params == '') return(if (label) list(label = unnamed_chunk()))
 
   res = withCallingHandlers(
     eval(parse_only(paste('alist(', quote_label(params), ')'))),
@@ -154,11 +171,11 @@ parse_params = function(params) {
     stop('invalid chunk options: ', params,
          "\n(all options must be of the form 'tag=value' except the chunk label)")
   if (is.null(res$label)) {
-    if (n == 0L) res$label = unnamed_chunk() else names(res)[idx] = 'label'
+    if (n == 0L) res$label = '' else names(res)[idx] = 'label'
   }
   if (!is.character(res$label))
     res$label = gsub(' ', '', as.character(as.expression(res$label)))
-  if (identical(res$label, '')) res$label = unnamed_chunk()
+  if (identical(res$label, '')) res$label = if (label) unnamed_chunk()
   res
 }
 
@@ -175,8 +192,102 @@ quote_label = function(x) {
   x
 }
 
+# comment characters for various languages
+comment_chars = list(
+  `#` = c('awk', 'bash', 'coffee', 'gawk', 'julia', 'octave', 'perl', 'powershell', 'python', 'r', 'ruby', 'sed', 'stan'),
+  '//' = c('asy', 'cc', 'csharp', 'd3', 'dot', 'fsharp', 'go', 'groovy', 'java', 'js', 'node', 'Rcpp', 'sass', 'scala'),
+  `%` = c('matlab', 'tikz'),
+  `/* */` = c('c', 'css'),
+  `* ;` = c('sas'),
+  `--` = c('haskell', 'lua', 'mysql', 'psql', 'sql'),
+  `!` = c('fortran', 'fortran95'),
+  `*` = c('stata')
+)
+# reshape it using the language name as the index, i.e., from list(char = lang)
+# to list(lang = char)
+comment_chars = local({
+  res = list()
+  for (i in names(comment_chars)) {
+    chars = comment_chars[[i]]
+    res = c(res, setNames(rep(list(strsplit(i, ' ')[[1]]), length(chars)), chars))
+  }
+  res[order(names(res))]
+})
+
+# partition YAML (chunk options) from a code chunk of the following form:
+# #| echo: true
+# #| foo: bar
+# 1 + 1
+#
+# we also allow for traditional CSV syntax, e.g.,
+# #| echo = TRUE, fig.width = 8
+partition_chunk = function(engine, code) {
+
+  res = list(yaml = NULL, src = NULL, code = code)
+  # mask out empty blocks
+  if (length(code) == 0) return(res)
+
+  char = comment_chars[[engine]] %n% '#'
+  s1 = paste0(char[[1]], '| ')
+  s2 = ifelse(length(char) > 1, char[[2]], '')
+
+  # check for option comments
+  i1 = startsWith(code, s1)  # [start|end]sWith() requires R >= 3.3.0
+  i2 = endsWith(trimws(code, 'right'), s2)
+  # if "commentChar| " is not found, try "#| " instead
+  if (!i1[1] && !identical(char, '#')) {
+    s1 = '#| '; s2 = ''
+    i1 = startsWith(code, s1); i2 = TRUE
+  }
+  m = i1 & i2
+
+  # has to have at least one matched line at the beginning
+  if (!m[[1]]) return(res)
+
+  # divide into yaml and code
+  if (all(m)) {
+    src = code
+    code = NULL
+  } else {
+    src = head(code, which.min(m) - 1)
+    code = tail(code, -length(src))
+  }
+
+  # trim right
+  if (any(i2)) src = trimws(src, 'right')
+
+  # extract meta from comments, then parse it
+  meta = substr(src, nchar(s1) + 1, nchar(src) - nchar(s2))
+  # see if the metadata looks like YAML or CSV
+  if (grepl('^[^ :]+:($|\\s)', meta[1])) {
+    meta = yaml::yaml.load(meta, eval.expr = TRUE)
+    if (!is.list(meta) || length(names(meta)) == 0) {
+      warning('Invalid YAML option format in chunk: \n', one_string(meta), '\n')
+      meta = list()
+    }
+  } else {
+    meta = parse_params(paste(meta, collapse = ''), label = FALSE)
+  }
+
+  # normalize field name 'id' to 'label' if provided
+  meta$label = unlist(meta[c('label', 'id')])[1]
+  meta$id = NULL
+  # convert any option with fig- into fig. and out- to out.
+  names(meta) = sub('^(fig|out)-', '\\1.', names(meta))
+
+  # extract code
+  if (length(code) > 0 && is_blank(code[[1]])) {
+    code = code[-1]
+    src = c(src, '')
+  }
+
+  list(options = meta, src = src, code = code)
+}
+
 print.block = function(x, ...) {
   params = x$params
+  # don't show internal options for quarto
+  for (i in attr(params, 'quarto_options')) params[[i]] = NULL
   cat('label:', params$label)
   if (length(params) > 1L) {
     cat(' (with options) \n')
@@ -398,18 +509,74 @@ parse_chunk = function(x, rc = knit_patterns$get('ref.chunk')) {
   unlist(x, use.names = FALSE)
 }
 
-# filter chunk.end lines that don't actually end a chunk
-filter_chunk_end = function(chunk.begin, chunk.end) {
-  in.chunk = FALSE
-  fun = function(is.begin, is.end) {
-    if (in.chunk && is.end) {
-      in.chunk <<- FALSE
-      return(TRUE)
+# split text lines into groups of code and text chunks
+group_indices = function(chunk.begin, chunk.end, lines = NA, is.md = FALSE) {
+  in.chunk = FALSE  # whether inside a chunk now
+  pattern.end = NA  # the expected chunk end pattern (derived from header)
+  b = NA  # the last found chunk header
+  # TODO: for now we only disallow unmatched delimiters during R CMD check
+  # that's not running on CRAN; we will fully disallow it in the future (#2057)
+  signal = if (is_R_CMD_check() && !(is_cran() || is_bioc())) stop2 else warning2
+  g = NA  # group index: odd - text; even - chunk
+  fun = function(is.begin, is.end, line, i) {
+    if (i == 1) {
+      g <<- if (is.begin) {
+        in.chunk <<- TRUE
+        b <<- i
+        0
+      } else 1
+      return(g)
     }
-    if (!in.chunk && is.begin) in.chunk <<- TRUE
-    FALSE
+    # begin of another chunk is found while the previous chunk is not complete yet
+    if (in.chunk && is.begin) {
+      if (!is.md || match_chunk_begin(pattern.end, line)) {
+        g <<- g + 2  # same amount of ` as previous chunk, so should be a new chunk
+        if (is.md) b <<- i
+      }  # otherwise ignore the chunk header
+      return(g)
+    }
+    if (in.chunk && is.end && match_chunk_end(pattern.end, line, i, b, lines, signal)) {
+      in.chunk <<- FALSE
+      g <<- g + 1
+      return(g - 1)  # don't use incremented g yet; use it in the next step
+    }
+    if (!in.chunk && is.begin) {
+      in.chunk <<- TRUE
+      if (is.md) {
+        pattern.end <<- sub('(^[\t >]*```+).*', '^\\1\\\\s*$', line)
+        b <<- i
+      }
+      g <<- g + 2 - g%%2  # make sure g is even
+    }
+    g
   }
-  mapply(fun, chunk.begin, chunk.end)
+  mapply(fun, chunk.begin, chunk.end, lines, seq_along(lines))
+}
+
+match_chunk_begin = function(pattern.end, x, pattern = '^\\1\\\\{') {
+  grepl(gsub('^([^`]*`+).*', pattern, pattern.end), x)
+}
+
+match_chunk_end = function(pattern, line, i, b, lines, signal = stop) {
+  if (is.na(pattern) || grepl(pattern, line)) return(TRUE)
+  n = length(lines)
+  # if the exact match was not found, look ahead to see if there is another
+  # chunk end that is an exact match before the next chunk begin
+  if (i < n && length(k <- grep(pattern, lines[(i + 1):n]))) {
+    k = k[1]
+    if (k == 1) return(FALSE)  # the next line is real chunk end
+    # no other chunk headers before the new next exact chunk end
+    if (!any(match_chunk_begin(pattern, lines[i + 1:(k - 1)], '^\\1`*\\\\{')))
+      return(FALSE)
+  }
+  signal(
+    'The closing backticks on line ', i, ' ("', line, '") in ', current_input(),
+    ' do not match the opening backticks "',
+    gsub('\\^(\\s*`+).*', '\\1', pattern), '" on line ', b, '. You are recommended to ',
+    'fix either the opening or closing delimiter of the code chunk to use exactly ',
+    'the same numbers of backticks and same level of indentation (or blockquote).'
+  )
+  TRUE
 }
 
 #' Get all chunk labels in a document
