@@ -7,21 +7,16 @@
 #' @name vignette_engines
 #' @note If you use the \code{knitr::rmarkdown} engine, please make sure that
 #'   you put \pkg{rmarkdown} in the \samp{Suggests} field of your
-#'   \file{DESCRIPTION} file. Also make sure the executables \command{pandoc}
-#'   and \command{pandoc-citeproc} can be found by \pkg{rmarkdown} during
-#'   \command{R CMD build}. If you build your package from RStudio, this is
-#'   normally not a problem. If you build the package outside RStudio, run
-#'   \command{which pandoc} and \command{which pandoc-citeproc} in the terminal
-#'   (or \code{Sys.which('pandoc')} and \code{Sys.which('pandoc-citeproc')} in
-#'   R) to check if \command{pandoc} and \command{pandoc-citeproc} can be found.
-#'   If you use Linux, you may make symlinks to the Pandoc binaries in RStudio:
-#'   \url{https://pandoc.org/installing.html}, or
-#'   install \command{pandoc} and \command{pandoc-citeproc} separately.
+#'   \file{DESCRIPTION} file. Also make sure \command{pandoc} is available
+#'   during \command{R CMD build}. If you build your package from RStudio, this
+#'   is normally not a problem. If you build the package outside RStudio, run
+#'   \code{rmarkdown::find_pandoc()} in an R session to check if Pandoc can be
+#'   found.
 #'
 #'   When the \pkg{rmarkdown} package is not installed or not available, or
-#'   \command{pandoc} or \command{pandoc-citeproc} cannot be found, the
-#'   \code{knitr::rmarkdown} engine will fall back to the \code{knitr::knitr}
-#'   engine, which uses R Markdown v1 based on the \pkg{markdown} package.
+#'   \command{pandoc} cannot be found, the \code{knitr::rmarkdown} engine will
+#'   fall back to the \code{knitr::knitr} engine, which uses R Markdown v1 based
+#'   on the \pkg{markdown} package.
 #' @examples library(knitr)
 #' vig_list = tools::vignetteEngine(package = 'knitr')
 #' str(vig_list)
@@ -33,18 +28,21 @@ NULL
 vweave = function(file, driver, syntax, encoding = 'UTF-8', quiet = FALSE, ...) {
   {
     on.exit({opts_chunk$restore(); knit_hooks$restore()}, add = TRUE)
-    oopts = options(markdown.HTML.header = NULL); on.exit(options(oopts), add = TRUE)
+    oopts = options(markdown.HTML.header = NULL, knitr.knit2html.force_v1 = TRUE)
+    on.exit(options(oopts), add = TRUE)
   }
   opts_chunk$set(error = FALSE)  # should not hide errors
   knit_hooks$set(purl = hook_purl)  # write out code while weaving
-  (if (grepl('\\.[Rr]md$', file)) knit2html_v1 else if (grepl('\\.[Rr]rst$', file)) knit2pandoc else knit)(
+  (if (grepl('\\.[Rr]md$', file)) knit2html else if (grepl('\\.[Rr]rst$', file)) knit2pandoc else knit)(
     file, encoding = encoding, quiet = quiet, envir = globalenv(), ...
   )
 }
 
 vtangle = function(file, ..., encoding = 'UTF-8', quiet = FALSE) {
-  if (is_R_CMD_check()) {
-    file = with_ext(file, 'R')
+  # a hack to generate an empty R script to cheat R CMD check because purl() is
+  # not reliable (#2052, #2036)
+  if (xfun::is_R_CMD_check() && !file.exists(with_ext(file, 'Rout.save'))) {
+    file = with_ext(file, '.R')
     file.create(file)
     return(file)
   }
@@ -84,29 +82,30 @@ vtangle_empty = function(file, ...) {
   return()
 }
 
+# when neither Pandoc nor markdown is available, just silently skip the vignette
+vweave_empty = function(file, ..., .reason = 'Pandoc') {
+  out = with_ext(file, 'html')
+  writeLines(sprintf('The vignette could not be built because %s is not available.', .reason), out)
+  out
+}
+
 register_vignette_engines = function(pkg) {
   # the default engine
   vig_engine('knitr', vweave, '[.]([rRsS](nw|tex)|[Rr](md|html|rst))$')
   vig_engine('docco_linear', vweave_docco_linear, '[.][Rr](md|markdown)$')
   vig_engine('docco_classic', vweave_docco_classic, '[.][Rr]mk?d$')
-  vig_engine('rmarkdown', function(...) if (has_package('rmarkdown')) {
+  vig_engine('rmarkdown', function(...) {
+    if (is_cran_check() && !has_package('rmarkdown'))
+      return(vweave_empty(..., .reason = 'rmarkdown'))
     if (pandoc_available()) {
       vweave_rmarkdown(...)
     } else {
-      warning(
-        'Pandoc (>= 1.12.3) and/or pandoc-citeproc not available. ',
-        'Falling back to R Markdown v1.'
+      (if (is_R_CMD_check()) message else stop2)(
+        'Pandoc is required to build R Markdown vignettes but not available. ',
+        'Please make sure it is installed.'
       )
-      vweave(...)
+      if (has_package('markdown')) vweave(...) else vweave_empty(...)
     }
-  } else {
-    # TODO: no longer allow fallback to R Markdown v1
-    (if (xfun::is_CRAN_incoming()) stop2 else warning)(
-      'The vignette engine knitr::rmarkdown is not available because the rmarkdown ',
-      'package is not available. Did you forget to add it to Suggests in DESCRIPTION? ',
-      'Please see https://github.com/yihui/knitr/issues/1864 for more information.'
-    )
-    vweave(...)
   }, '[.][Rr](md|markdown)$')
   # vignette engines that disable tangle
   vig_list = tools::vignetteEngine(package = 'knitr')
@@ -153,14 +152,10 @@ knit_filter = function(ifile, encoding = 'UTF-8') {
   p = detect_pattern(x, tolower(file_ext(ifile)))
   if (is.null(p)) return(x)
   p = all_patterns[[p]]; p1 = p$chunk.begin; p2 = p$chunk.end
-  i1 = grepl(p1, x)
-  i2 = filter_chunk_end(i1, grepl(p2, x))
-  m = numeric(n)
-  m[i1] = 1; m[i2] = 2  # 1: code; 2: text
-  if (m[1] == 0) m[1] = 2
-  for (i in seq_len(n - 1)) if (m[i + 1] == 0) m[i + 1] = m[i]
-  x[m == 1 | i2] = ''
-  x[m == 2] = stringr::str_replace_all(x[m == 2], p$inline.code, '')
+  m = group_indices(grepl(p1, x), grepl(p2, x))
+  i = m %% 2 == 0
+  x[i] = ''  # remove code chunks
+  x[!i] = stringr::str_replace_all(x[!i], p$inline.code, '')  # remove inline code
   structure(x, control = '-H -t')
 }
 
