@@ -148,25 +148,28 @@ output_asis = function(x, options) {
   is_blank(x) || options$results == 'asis'
 }
 
-# path relative to dir of the input file
-input_dir = function(ignore_root = FALSE) {
+# the working directory: use root.dir if specified, otherwise the dir of the
+# input file unless knitr.use.cwd = TRUE
+input_dir = function() {
   root = opts_knit$get('root.dir')
-  # LyX is a special case: the input file is in tempdir, and we should use
-  # root.dir as the real input dir (#809)
-  if (is_lyx()) return(root)
-  if (ignore_root) {
-    .knitEnv$input.dir %n% '.'
-  } else {
-    root %n% (if (!getOption('knitr.use.cwd', FALSE)) .knitEnv$input.dir) %n% '.'
+  root %n% (if (!getOption('knitr.use.cwd', FALSE)) .knitEnv$input.dir) %n% '.'
+}
+
+# detect if running on CRAN (assuming that CRAN does not set CI or
+# NOT_CRAN=true); or set R_CRANDALF=true (is cran) or false (not cran)
+is_cran = function() {
+  x = Sys.getenv('R_CRANDALF', NA)
+  if (!is.na(x)) tolower(x) == 'true' else {
+    !any(tolower(Sys.getenv(c('CI', 'NOT_CRAN'))) == 'true')
   }
 }
 
-is_lyx = function() {
-  args = commandArgs(TRUE)
-  if (length(args) < 4) return(FALSE)
-  grepl('[.]Rnw$', args[1]) && !is.na(Sys.getenv('LyXDir', NA))
+is_cran_check = function() {
+  is_cran() && is_R_CMD_check()
 }
 
+is_bioc = function() Sys.getenv('BBS_HOME') != ''
+
 # round a number to getOption('digits') decimal places by default, and format()
 # it using significant digits if the option knitr.digits.signif = TRUE
 round_digits = function(x) {
@@ -234,7 +237,8 @@ tikz_dict = function(path) {
   paste(sans_ext(basename(path)), 'tikzDictionary', sep = '-')
 }
 
-# compatibility with Sweave and old beta versions of knitr
+# initially for compatibility with Sweave and old beta versions of knitr
+# but now also place to tweak default options
 fix_options = function(options) {
   options = as.strict_list(options)
 
@@ -256,8 +260,12 @@ fix_options = function(options) {
 
   # cache=TRUE -> 3; FALSE -> 0
   if (is.logical(options$cache)) options$cache = options$cache * 3
-  # non-R code should not use cache=1,2
-  if (options$engine != 'R') options$cache = (options$cache > 0) * 3
+  if (options$engine != 'R') {
+    # non-R code should not use cache = 1, 2
+    options$cache = (options$cache > 0) * 3
+    # error numbers only make sense to R engine, so 0 -> TRUE; 1, 2 -> FALSE
+    if (is.numeric(options$error)) options$error = options$error == 0
+  }
 
   options$eval = unname(options$eval)
 
@@ -297,12 +305,17 @@ fix_options = function(options) {
     }
   }
 
+  # adjust some options when collapse is TRUE
   if (options$collapse) {
     options[unlist(lapply(
       c('class.', 'attr.'), paste0, c('output', 'message', 'warning', 'error')
     ))] = NULL
   }
 
+  # change default of value conditionally
+  if (identical(options$strip.white, I(TRUE)))
+    options$strip.white = !options$collapse
+
   options
 }
 
@@ -355,7 +368,8 @@ is_latex_output = function() {
 #'   provided, \code{is_html_output()} uses \code{pandoc_to()}, and
 #'   \code{pandoc_to()} returns the output format name.
 #' @param excludes A character vector of output formats that should not be
-#'   considered as HTML format.
+#'   considered as HTML format. Options are: markdown, epub, html, html4, html5,
+#'   revealjs, s5, slideous, slidy, and gfm.
 #' @rdname output_type
 #' @export
 is_html_output = function(fmt = pandoc_to(), excludes = NULL) {
@@ -554,6 +568,10 @@ print_knitlog = function() {
 line_count = function(x) stringr::str_count(x, '\n') + 1L
 
 has_package = function(pkg) loadable(pkg, FALSE)
+has_packages = function(pkgs) {
+  for (p in pkgs) if (!has_package(p)) return(FALSE)
+  TRUE
+}
 
 # if LHS is NULL, return the RHS
 `%n%` = function(x, y) if (is.null(x)) y else x
@@ -564,6 +582,18 @@ merge_list = function(x, y) {
   x
 }
 
+# find a token in a template, and replace it with a value
+insert_template = function(text, token, value, ignore = FALSE) {
+  if (is.null(value)) return(text)
+  i = grep(token, text); n = length(i)
+  if (n > 1) stop("There are multiple tokens in the template: '", token, "'")
+  if (n == 0) {
+    if (ignore) return(text)
+    stop("Couldn't find the token '", token, "' in the template.")
+  }
+  append(text, value, i)
+}
+
 # paths of all figures
 all_figs = function(options, ext = options$fig.ext, num = options$fig.num) {
   unlist(lapply(ext, fig_path, options = options, number = seq_len(num)))
@@ -599,7 +629,8 @@ escape_latex = function(x, newlines = FALSE, spaces = FALSE) {
   x = gsub('~', '\\\\textasciitilde{}', x)
   x = gsub('\\^', '\\\\textasciicircum{}', x)
   if (newlines) x = gsub('(?<!\n)\n(?!\n)', '\\\\\\\\', x, perl = TRUE)
-  if (spaces) x = gsub('  ', '\\\\ \\\\ ', x)
+  # when there are consecutive spaces, escape each of them except the first one
+  if (spaces) x = gsub('(?<= ) ', '\\\\ ', x, perl = TRUE)
   x
 }
 
@@ -765,7 +796,7 @@ knit_handlers = function(fun, options) {
   if (!is.function(fun)) fun = function(x, ...) {
     res = withVisible(knit_print(x, ...))
     # indicate the htmlwidget result with a special class so we can attach
-    # the figure caption to it later in wrap.knit_asis
+    # the figure caption to it later in sew.knit_asis
     if (inherits(x, 'htmlwidget'))
       class(res$value) = c(class(res$value), 'knit_asis_htmlwidget')
     if (res$visible) res$value else invisible(res$value)
@@ -811,9 +842,10 @@ create_label = function(..., latex = FALSE) {
 #'
 #' If the length of the input \code{words} is smaller than or equal to 1,
 #' \code{words} is returned. When \code{words} is of length 2, the first word
-#' and second word are combined using the \code{and} string. When the length is
-#' greater than 2, \code{sep} is used to separate all words, and the \code{and}
-#' string is prepended to the last word.
+#' and second word are combined using the \code{and} string, or if blank,
+#' \code{sep} if is used. When the length is greater than 2, \code{sep} is used
+#' to separate all words, and the \code{and} string is prepended to the last
+#' word.
 #' @param words A character vector.
 #' @param sep Separator to be inserted between words.
 #' @param and Character string to be prepended to the last word.
@@ -835,7 +867,7 @@ combine_words = function(
   if (n == 0) return(words)
   words = paste0(before, words, after)
   if (n == 1) return(rs(words))
-  if (n == 2) return(rs(paste(words, collapse = and)))
+  if (n == 2) return(rs(paste(words, collapse = if (is_blank(and)) sep else and)))
   if (oxford_comma && grepl('^ ', and) && grepl(' $', sep)) and = gsub('^ ', '', and)
   words[n] = paste0(and, words[n])
   # combine the last two words directly without the comma
@@ -849,6 +881,10 @@ combine_words = function(
 warning2 = function(...) warning(..., call. = FALSE)
 stop2 = function(...) stop(..., call. = FALSE)
 
+warn_options_unsupported = function(option, to) {
+  warning2('Chunk option ', option, ' is not supported for ', to, ' output')
+}
+
 raw_markers = c('!!!!!RAW-KNITR-CONTENT', 'RAW-KNITR-CONTENT!!!!!')
 
 #' @export
@@ -957,8 +993,6 @@ raw_latex = function(x, ...) raw_block(x, 'latex', ...)
 #' @export
 raw_html = function(x, ...) raw_block(x, 'html', ...)
 
-trimws = function(x) gsub('^\\s+|\\s+$', '', x)
-
 optipng = function(...) {
   warning2('knitr:::optipng() has been deprecated; please use xfun::optipng()')
   xfun::optipng(...)
@@ -1016,6 +1050,3 @@ make_unique = function(x) {
 #' browseURL('logo.html') # you can check its HTML source
 #' }
 image_uri = function(f) xfun::base64_uri(f)
-
-# TODO: remove this function after the next version of bookdown is on CRAN
-is_abs_path = function(...) xfun::is_abs_path(...)
